/*
 * Copyright (C) 2012-2013 Facebook, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.facebook.nifty.client;

import com.facebook.nifty.core.NettyConfigBuilderBase;
import com.google.common.base.Preconditions;
import com.google.inject.Inject;
import org.jboss.netty.channel.socket.nio.NioSocketChannelConfig;

import java.lang.reflect.Proxy;

/*
 * Hooks for configuring various parts of Netty.
 */
public class NettyClientConfigBuilder extends NettyConfigBuilderBase
{
    // The constants come directly from Netty but are private in Netty. We need these default
    // values to call the NioClientSocketChannelFactory constructor with a custom timer.
    private static final int DEFAULT_BOSS_THREAD_COUNT = 1;
    private static final int DEFAULT_WORKER_THREAD_COUNT = Runtime.getRuntime().availableProcessors() * 2;
    private int bossThreadCount = DEFAULT_BOSS_THREAD_COUNT;
    private int workerThreadCount = DEFAULT_WORKER_THREAD_COUNT;
    private String name = "";

    private final NioSocketChannelConfig socketChannelConfig = (NioSocketChannelConfig) Proxy.newProxyInstance(
            getClass().getClassLoader(),
            new Class<?>[]{NioSocketChannelConfig.class},
            new Magic("")
    );

    @Inject
    public NettyClientConfigBuilder()
    {
        // Thrift turns TCP_NODELAY by default, and turning it off can have latency implications
        // so let's turn it on by default as well. It can still be switched off by explicitly
        // calling setTcpNodelay(false) after construction.
        getSocketChannelConfig().setTcpNoDelay(true);
    }

    public NioSocketChannelConfig getSocketChannelConfig()
    {
        return socketChannelConfig;
    }

    public NettyClientConfigBuilder setNiftyBossThreadCount(int bossThreadCount)
    {
        this.bossThreadCount = bossThreadCount;
        return this;
    }

    public int getNiftyBossThreadCount()
    {
        return bossThreadCount;
    }

    public NettyClientConfigBuilder setNiftyWorkerThreadCount(int workerThreadCount)
    {
        this.workerThreadCount = workerThreadCount;
        return this;
    }

    public int getNiftyWorkerThreadCount()
    {
        return workerThreadCount;
    }

    public NettyClientConfigBuilder setNiftyName(String name)
    {
        Preconditions.checkNotNull(name, "name is null");
        this.name = name;
        return this;
    }

    public String getNiftyName()
    {
        return name;
    }

}
